# use Pyglet ver. 2 or later
import numpy as np
import random
import math
import pyglet
from pyglet.gl import *

def prep_laplacian_op(n,m):
    A = np.zeros((n,m))
    for p in range(0,n):
        xi = 2*math.pi*p/n if p<=n//2 else 2*math.pi*(p-n)/n
        for q in range(0,m):
            eta = 2*math.pi*q/m if q<=m//2 else 2*math.pi*(q-m)/m
            A[p,q] = -xi*xi-eta*eta
    return A
            
def laplacian(a, op):
    a2 = np.fft.fft2(a)
    a2 = op * a2
    return np.fft.ifft2(a2).real

###

N = 128

p_a = 0.0
p_b = 1.2
p_c = 0.4

D_u = 0.2
D_v = 4.0

t = 0
dt = 0.001

op = prep_laplacian_op(N,N)

u = np.random.uniform(low=0.0, high=1.0, size=(N,N))
v = np.random.uniform(low=0.0, high=1.0, size=(N,N))

SCALE = 4
config = pyglet.gl.Config(double_buffer=True)
win = pyglet.window.Window(N*SCALE,N*SCALE,config=config,resizable=False)
win.set_caption('Fish Skin Pattern')
batch = pyglet.graphics.Batch()

def set_color(x):
    r = 100 + int((1 + math.cos(math.pi*x))*64)
    g = 100 + int((1 + math.cos(math.pi*x*0.125))*64)
    b = 128 + int((1 + math.sin(math.pi*x*0.3333333333333333))*64)
    return (r,g,b)

rects = np.empty(shape=(N,N), dtype='object')
for i in range(N):
    for j in range(N):
        rect = pyglet.shapes.Rectangle(x=i*SCALE, y=j*SCALE, width=SCALE, height=SCALE, batch=batch)
        rect.color = set_color(u[i,j])
        rects[i,j] = rect

@win.event
def on_draw():
    win.clear()
    glClearColor(0,0,0,1)
    for i in range(N):
        for j in range(N):
            rect = rects[i,j]
            rect.color = set_color(u[i,j])
    batch.draw()
    str = 'T={0:.3f}'.format(t)
    label = pyglet.text.Label(str,
                              color=(255,255,255,255),
                              font_size=12, x=10, y=10)
    label.draw()

def update(deltat):
    global u,v,t
    for cnt in range(100):
        dudt = D_u * laplacian(u,op) + p_a - p_b*u + u*u/(v*(1.0+p_c*u*u))
        dvdt = D_v * laplacian(v,op) + u*u - v
        u = u + dudt*dt
        v = v + dvdt*dt
        t = t + dt

pyglet.clock.schedule_interval(update, 1/20.0)
pyglet.app.run()
